# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Contact: timhessels@hotmail.com
Repository: https://github.com/TimHessels/watertools
Module: Collect/L30

Description:
This module downloads the Harmonized L30 data.
"""

from .Harmonized import main as Harmonized

__all__ = ['Harmonized']

__version__ = '0.1'
