# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Module: General

Description:
This module consists of the general functions that are used in the WA+ toolbox
"""

from watertools.General import data_conversions, raster_conversions

__all__ = ['data_conversions','raster_conversions']

__version__ = '0.1'